/**
 * Options for the mock server
 */
export interface PactV3Options {
    /**
     * Directory to write the pact file to
     */
    dir: string;
    /**
     * Consumer name
     */
    consumer: string;
    /**
     * Provider name
     */
    provider: string;
    /**
     * If the mock server should handle CORS pre-flight requests. Defaults to false
     */
    cors?: boolean;
    /**
     * Port to run the mock server on. Defaults to a random port
     */
    port?: number;
}
export interface V3ProviderState {
    description: string;
    parameters?: any;
}
export interface V3Request {
    method?: string;
    path?: string;
    query?: {
        [param: string]: string;
    };
    headers?: {
        [header: string]: string;
    };
    body?: any;
}
export interface V3Response {
    status: number;
    headers?: {
        [header: string]: string;
    };
    body?: any;
}
export interface V3MockServer {
    port: number;
    url: string;
    id: string;
}
export declare class PactV3 {
    private opts;
    private states;
    private pact;
    constructor(opts: PactV3Options & {});
    given(providerState: string, parameters?: any): PactV3;
    uponReceiving(desc: string): PactV3;
    withRequest(req: V3Request): PactV3;
    withRequestBinaryFile(req: V3Request, contentType: string, file: string): PactV3;
    withRequestMultipartFileUpload(req: V3Request, contentType: string, file: string, part: string): PactV3;
    willRespondWith(res: V3Response): PactV3;
    withResponseBinaryFile(res: V3Response, contentType: string, file: string): PactV3;
    withResponseMultipartFileUpload(req: V3Response, contentType: string, file: string, part: string): PactV3;
    executeTest<T>(testFn: (mockServer: V3MockServer) => Promise<T>): Promise<T>;
}
